/*
 * Ben NanoNote and EM027AS012 E-Paper Display Adapter Board communication.
 *
 * Copyright (C) 2013 Paul Boddie
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ubb/ubb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include "images.h"
#include "Display_Controller.h"
#include "bsp.h"

typedef enum {
    FALSE=0,
    TRUE=1
} bool;

/**
 * Handle termination of the process.
 */
void shutdown(int signum)
{
    printf("Closing...\n");
    ubb_close(0);
    exit(1);
}

/* Main program and basic initialisation. */

int main(int argc, char *argv[])
{
    uint8_t *current_image, *next_image, blank_image[176][33], image_number, image_number_arg;
    bool blank_screen;

    signal(SIGINT, &shutdown);

    if (ubb_open(0) < 0) {
        perror("ubb_open");
        return 1;
    }

    ubb_power(1);
    printf("Power on.\n");

    /* Need to initialise the board. */

    bsp_init();
    printf("Initialised.\n");

    /* Initialise an array containing the blank image. */

    memset((uint8_t*) &blank_image[0][0], 0xff, 176*33);

    /* Determine whether to blank the screen or leave an image on it. */

    blank_screen = (argc > 2) && (strcmp(argv[1], "--blank") == 0);
    image_number_arg = blank_screen ? 2 : 1;

    if ((argc < 2) || (!isdigit(argv[image_number_arg][0])))
    {
        printf("Usage: %s [ --blank ] <image number>\n", argv[0]);
        ubb_close(0);
        return 1;
    }
    else
    {
        image_number = atoi(argv[image_number_arg]);

        if (image_number > MAX_IMAGE)
        {
            printf("Image number cannot be greater than %d.\n", MAX_IMAGE);
            ubb_close(0);
            return 1;
        }

        /* When blanking, indicate an image presumably on screen already. */

        if (blank_screen)
        {
            next_image = (uint8_t*) &blank_image[0][0];
            current_image = (uint8_t *) &image_data[image_number][0][0];
            printf("Set blank image.\n");
        }

        /* When leaving an image on screen, indicate that the screen starts off
           being blank. */

        else
        {
            next_image = (uint8_t *) &image_data[image_number][0][0];
            current_image = (uint8_t*) &blank_image[0][0];
            printf("Set first image.\n");
        }
    }

    /* Call epd_DisplayImg with the new image and current image. */

    epd_DisplayImg(DISPLAY_IN_USE, next_image, current_image);
    printf("Updated image.\n");

    /* Detach from the board. */

    printf("Closing...\n");
    ubb_close(0);

    return 0;
}
